import express, {Request, Response} from 'express';
import {model, Schema, HydratedDocument, Types} from 'mongoose';
import multer from 'multer';
import fs from 'fs';
import path from 'path';
import {IArtist} from './artist';

const upload = multer({dest: 'uploads/albums'});
const router = express.Router();

export interface IAlbum {
	name: string;
	albumImage: File;
	year: number;
	artist: Types.ObjectId;
}

const Album = model<IAlbum>(
	'Album',
	new Schema({
		artist: {
			type: Schema.Types.ObjectId,
			ref: 'Artist',
		},
		name: String,
		year: Number,
		albumImage: {
			data: Buffer,
			contentType: String,
		},
	})
);

router.get('/', async (req: Request, res: Response) => {
	if (!req.query.artist) {
		try {
			const albums = await Album.find().populate<{artist: IArtist}>('artist');
			res.send(albums);
			return;
		} catch (err: unknown) {
			res.send('Could not get albums');
		}
	}
	try {
		const artistsAlbums = await Album.find({artist: req.query.artist});
		res.send(artistsAlbums);
	} catch (err: unknown) {
		res.send(`Could not find albums of ${req.query.artist} `);
	}
});

router.get('/:id', async (req: Request, res: Response) => {
	try {
		const album = await Album.findById(req.params.id).populate<{
			artist: IArtist;
		}>('artist');
		if (album === null) {
			res.send('Nothing such album found');
		}
		res.send(album);
	} catch (err: unknown) {
		res.send('No such artist exists');
	}
});

router.post(
	'/',
	upload.single('albumImage'),
	async (req: Request, res: Response) => {
		if (!req.body.name || !req.file || !req.body.year || !req.body.artist) {
			res.send('Name, year, artist or albumimage is required');
			return;
		}
		try {
			const album: HydratedDocument<IAlbum> = new Album({
				name: req.body.name,
				albumImage: {
					data: fs.readFileSync(
						path.join('uploads/albums/' + req.file?.filename)
					),
					contentType: 'image',
				},
				year: req.body.year,
				artist: req.body.artist,
			});
			await album.save();

			res.send('Sucessfully saved');
		} catch (err: unknown) {
			res.send('Could not post album');
		}
	}
);

export {router as albumRouter};
